#!/bin/sh
# the next line restarts using wish \
# replace that button with a menu bar entry... 
# the next line restarts using wish \
exec wish "$0" "$@"


# Init stuff

eval destroy [winfo child .]



# Instructions to the window manager 

wm title . "Fortune/TK"
wm iconname . "Fortune"
wm maxsize . 80 8 

# Fonts

set font "Helvetica 12"
set MenuFont "Helvetica 11 bold"
set TextBoxFont "Times 12"
set AlertFont "Times 14"


# Initialising all variables 

set w .base
set addedinfo "(extra info)"
set tip1 "  Press Enter, Space or F1 for a new fortune quote"
set menustatus $tip1
set debug "0"


# These variables make a real difference to the appearance of the thing and are
# changeable in later versions (saved as user options)

set buttonoptions n
#set fortunepath [auto_execok fortune] 
set fortunepath "/usr/games/fortune"
# Initialising old fortune list 

set lastfortune [list "You haven't seen any arrays yet!"]
lappend lastfortune "You haven't seen any arrays yet!"
set NumberOfFortunesSeen 1 
set currentindex 1

# Initialising main window

frame $w -relief flat 
pack $w -side bottom -fill y -anchor s


# Key Bindings
# The last ten fortunes
# ought to be reachable by arrow key. And it should give out useful 
# information of this kind using the status bar.

# Plus I definitely need to do some kind of reduced size port of fortune for
# people who aren't running unix and don't have it on their systems. That would
# be totally neat as almost anybody could then use this (less to install), even
# if less fortunes would be available for fortune-free people

# Keys that produce new fortunes

bind . <F1> {newFortune $w}
bind . <Return> {newFortune $w}
bind . <Key-space> {newFortune $w}
bind . <F2> {newFortune $w}

# Keys that kill the window

bind . <Control-c> {destroy .}
bind . <Control-q> {destroy .}

# The four arrow keys - yup, that's right, browser style navigation

bind . <Key-Right> {showLastFortune $w plusOne}
bind . <Key-Left> {showLastFortune $w minusOne}

#bind . <Tab> {}


# Menu event bindings

bind Menu <<MenuSelect>> {
    global $menustatus
    if {[catch {%W entrycget active -label} label]} {
    set label "   "
	set addedinfo "  "
    }
		
	set info $label
    set menustatus "$info"
    update idletasks
}



# Menu Designing

menu .topmenu -font $MenuFont -tearoff 0
set ma $w.topmenu
set m .topmenu.options
.topmenu add cascade -label "Options" -menu .topmenu.options -underline 0
menu .topmenu.options -tearoff 0
$m add command -label "Save this fortune to your favourites" -command {error "doesn't do anything right now"}
$m add command -label "Save to new file" -command {error "doesn't do anything
either, but it's hardly complex"}
$m add separator
$m add command -label "Configuration" -command {error "haven't got around to designing this to do anything yet"}
$m add separator
$m add command -label "Quit    C-q" -command "destroy ."
. configure -menu .topmenu

# Now for the search menu
# Yes, that's right, a search menu. you can search for a random quote on a
# particular topic. Or not so random. Maybe even forward and back buttons if I
# can be bothered to implement all that.

.topmenu  add cascade -label "Search" -menu .topmenu.search -underline 0
menu .topmenu.search -tearoff 0
.topmenu.search add command -label "Search by field" -command {error "Please
implement me"}
.topmenu.search add command -label "Search by keyword" -command {error "Please
implement me too"}

# The Add Fortune menu
.topmenu add cascade -label "Add fortune" -menu .topmenu.addfortune -underline 0 
menu .topmenu.addfortune -tearoff 0
.topmenu.addfortune add command -label "Add a fortune to the database" -command {set addedinfo "   Use this command to add a fortune to the official database files"} 
.topmenu.addfortune add command -label "Add a fortune to your personal file" -command {set addedinfo "   Use this command to add a fortune to your personal fortune file (doesn't require root access)"} 

# Now for the subject menu
.topmenu add cascade -label "Pick a subject" -menu .topmenu.subject -underline 0
menu .topmenu.subject -tearoff 0
.topmenu.subject add command -label "Quote subject preferences" -command {error
"Got to write a widget for this before it'll work"} 


# Now for the about: menu
.topmenu add cascade -label "Info" -menu .topmenu.about -underline 0
menu .topmenu.about -tearoff 0
.topmenu.about add command -label "About the licence" -command {
		list [source "fortune-licence.tcl"]
		update
		}
.topmenu.about add command -label "About this program" -command {
	     list [source "fortune-info.tcl"]
        update
	}
.topmenu.about add separator
.topmenu.about add command -label "Narcissism" -command {aboutBox}

# Now for a quick try at a status bar new fortune button
.topmenu add cascade -label "Fortune Navigation" -menu .topmenu.new -underline 0
menu .topmenu.new -tearoff 0
.topmenu.new add command -label "New fortune (Enter or F1)" -command {newFortune $w}
.topmenu.new add command -label "Last fortune (left arrow)" -command {showLastFortune $w minusOne}
.topmenu.new add command -label "Next fortune (right arrow)" -command {showLastFortune $w plusOne}
# Status bar frame

frame $w.statusBar
label $w.statusBar.lab -textvariable menustatus -relief sunken -bd 1 \
    -font -*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* -anchor w
label $w.statusBar.foo -width 8 -relief sunken -bd 1 \
    -font -*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* -anchor w
pack $w.statusBar.lab -side left -padx 2 -expand yes -fill both
pack $w.statusBar.foo -side left -padx 2
pack $w.statusBar -side bottom -fill x -pady 2


# Optional button frame, for people who like mice and approve of New Fortune
# buttons to clutter life up

if {$buttonoptions =="y"} {
	frame $w.buttons -relief flat
	pack $w.buttons -side bottom -fill y -anchor s 

	# Defining the optional button bar

    button $w.buttons.showNewFortune -text "Show new fortune" -command {
        newFortune $w
    }
    pack $w.buttons.showNewFortune -fill x
}



# Fortune-generating function

proc ::newFortunePath fpath {
	global fortunepath
	
}

proc ::newFortune w {
	global currentindex 
	global debug
	global fortunepath
	global lastfortune
	global NumberOfFortunesSeen
	set NumberOfFortunesSeen [incr NumberOfFortunesSeen 1] 
	set currentindex $NumberOfFortunesSeen
	$w.frame.text configure -state normal 
	$w.frame.text delete 1.0 end 
    set result [ exec $fortunepath]
	lappend lastfortune $result
   	
	 $w.frame.text insert 1.0 $result $w.text 
	$w.frame.text configure -state disabled 

	#Debug stuff
	if {$debug == 1} {
		puts stdout "Number of fortunes seen is $NumberOfFortunesSeen"
		puts stdout "Number of fortunes in list is [llength $lastfortune]"
		puts stdout "Currently on index number $currentindex"
	}
}

proc showLastFortune {w direction} {
	
	global currentindex 
	global debug
	global fortunepath
	global lastfortune
	global NumberOfFortunesSeen
	if {[string match $direction minusOne]} {
	   	if { $currentindex ==0} {
        	$w.frame.text configure -state normal
	       	$w.frame.text delete 1.0 end
     		$w.frame.text insert 1.0 {You can't go back any further! That was the first fortune you saw!}
       		$w.frame.text configure -state disabled

    	} else {

			incr currentindex -1
			$w.frame.text configure -state normal
			$w.frame.text delete 1.0 end
			$w.frame.text insert 1.0 [lindex $lastfortune $currentindex]
			$w.frame.text configure -state disabled
		}	
	} elseif {[string match $direction plusOne]} {
		if {$currentindex== $NumberOfFortunesSeen} {
				#should figure out how to make this do a newfortune properly
			   	set NumberOfFortunesSeen [incr NumberOfFortunesSeen 1]
 			   	set currentindex $NumberOfFortunesSeen
    			$w.frame.text configure -state normal
    			$w.frame.text delete 1.0 end
    			set result [ exec $fortunepath]
    			lappend lastfortune $result

     			$w.frame.text insert 1.0 $result $w.text
    			$w.frame.text configure -state disabled
		
				
			} else {	
				incr currentindex +1
				$w.frame.text configure -state normal 
            	$w.frame.text delete 1.0 end
            	$w.frame.text insert 1.0 [lindex $lastfortune $currentindex]
            	$w.frame.text configure -state disabled
			}
		}
	#Debug stuff
	if {$debug == 1} {
		puts stdout "Currently on index number $currentindex"
		puts stdout "Direction is: $direction"
	}
}


# The text box

frame $w.frame -relief flat 
pack $w.frame -side left -fill y -anchor center


# The Scrollbar

scrollbar $w.frame.scroller -command "$w.frame.text yview"
pack $w.frame.scroller -side right -fill y


# The actual text box

text $w.frame.text -relief sunken -state disabled  -font $TextBoxFont -bd 2 -width 80  -wrap word -yscrollcommand \
"$w.frame.scroller set" -setgrid 1  -height 10 
pack $w.frame.text -side top -pady 2 -anchor e



# insert a new fortune just to start the day off with
# sometime we should start it writing config files and reading them on startup
# so that it actually saves your config info


# Whoops! No fortune? What do we do? For now, we just put up a comment
# complaining that they haven't installed fortune.

if {[string length $fortunepath]} {

	newFortune $w
} else {
    global fortunepath
    global lastfortune
    global NumberOfFortunesSeen
    $w.frame.text configure -state normal
    $w.frame.text delete 1.0 end
	set fish {Oh dear,
It seems that this program can't find Fortune on your system!
No /usr/bin/fortune! 
Or c:\Fortune as it might be...  
Never mind, go to my site and download Fortune for People Who Don't Have The Real Thing, a reduced subset... or look for a port of the real thing to your platform. You never know :) 
http://www.typodemon.com}
	$w.frame.text insert 1.0 $fish $w.text
    $w.frame.text configure -state disabled

}

focus $w.frame.scroller 

# New window procedures, alert boxes, etc

proc aboutBox {} {
	global AlertFont
    message -icon info -type ok -title "About Widget Demo" -message \
"Fortune graphical interface\n\n\
Copyleft June 2000 by Em Tonkin. \n This software is released under the GNU
licence\n"
}

# The end
